package edu.berkeley.cs160.theflash.flashlight;

/* Based on AlarmController.java and DateWidgets1.java from API Demos */

import android.app.Activity;
import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.TimePickerDialog;
import android.content.Intent;
import android.os.Bundle;
import android.widget.TextView;
import android.widget.Button;
import android.widget.TimePicker;
import android.widget.Toast;
import android.widget.RemoteViews;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;


import java.util.Calendar;
import java.util.Timer;
import java.util.TimerTask;
import java.lang.String;

public class AlarmController extends Activity {

    // where we display the selected date and time
    public static TextView mDateDisplay;
    public static TextView mAlarmStatusDisplay;
    
    public static boolean alarmSet = false;
    public static NotificationManager mNotificationManager;
    
    // date and time
    public static int mHour;
    public static int mMinute;
    
    private int tmpHour;
    private int tmpMinute;
    
    public static String AMPM;
    
    // Timer used to reset
    public static Timer timer = new Timer();
    
    // for alarm
    Toast mToast;
    
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);

        // Turn off the title bar
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        
        mNotificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        
        setContentView(R.layout.alarm_controller);

        mDateDisplay = (TextView) findViewById(R.id.dateDisplay);
        mAlarmStatusDisplay = (TextView) findViewById(R.id.alarmStatusDisplay);

        Button pickTime = (Button) findViewById(R.id.pickTime);
        pickTime.setOnClickListener(new View.OnClickListener() {

            public void onClick(View v) {
            	if (alarmSet) {
            		tmpHour = mHour;
            		if (AMPM == "PM") {
            			tmpHour += 12;
            		}
            		tmpMinute = mMinute;
            	} else {
            		Calendar calendar = Calendar.getInstance();
            		tmpHour = calendar.get(Calendar.HOUR_OF_DAY);
            		tmpMinute = calendar.get(Calendar.MINUTE);
            	}
                new TimePickerDialog(AlarmController.this,
                        mTimeSetListener, "Set the time",
                        tmpHour, tmpMinute, false).show();
            }
        });
        
        Button turnOff = (Button) findViewById(R.id.turnOff);
        turnOff.setOnClickListener(new View.OnClickListener() {

            public void onClick(View v) {
            	/* Begin Alarm Code */
            	
                // Create the same intent that was scheduled.
                Intent intent = new Intent(AlarmController.this, OneShotAlarm.class);

                // And cancel the alarm.
                AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
                am.cancel(intent);

                // Tell the user about what we did.
                if (mToast != null) {
                    mToast.cancel();
                }
                if (alarmSet) {
                	mToast = Toast.makeText(AlarmController.this, R.string.alarm_unscheduled,
                			 Toast.LENGTH_SHORT);
                } else {
                	mToast = Toast.makeText(AlarmController.this, R.string.no_alarm_unscheduled,
                			 Toast.LENGTH_SHORT);
                }
                mToast.show();
                
                /* End Alarm Code */
                
                timer.cancel();
                timer = new Timer();
                alarmSet = false;
                mNotificationManager.cancel(R.layout.alarm_controller);
                updateDisplay();
            }
        });
        
        updateDisplay();
        
        //testing
        
      //set onKeyListener
       pickTime.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		//Pressing the a key
        		if(keyCode==KeyEvent.KEYCODE_CAMERA && event.getAction() == KeyEvent.ACTION_DOWN){
        			Intent intent = new Intent(AlarmController.this, Blinking.class);
    	            startActivity(intent);
    	            finish();
        			return true;
        		}else{
        			return false;
        		}
        	}
        });
       
     //set onKeyListener
       turnOff.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		//Pressing the a key
        		if(keyCode==KeyEvent.KEYCODE_CAMERA && event.getAction() == KeyEvent.ACTION_DOWN){
        			Intent intent = new Intent(AlarmController.this, Blinking.class);
    	            startActivity(intent);
    	            finish();
        			return true;
        		}else{
        			return false;
        		}
        	}
        });
    }

    public void updateDisplay() {
        if (alarmSet) {
        	mAlarmStatusDisplay.setText("Alarm is currently set for:");
            mDateDisplay.setText(
                    new StringBuilder()
                            .append(pad(mHour)).append(":")
                            .append(pad(mMinute))
                            .append(AMPM));
        } else {
        	mAlarmStatusDisplay.setText("Alarm is currently not set.");
        	mDateDisplay.setText("");
        }
    }

    private TimePicker.OnTimeSetListener mTimeSetListener =
            new TimePicker.OnTimeSetListener() {

                public void timeSet(TimePicker view, int hourOfDay, int minute) {                  
                    /* Begin Alarm Code */
                    
                    Intent intent = new Intent(AlarmController.this, OneShotAlarm.class);

                    Calendar calendar = Calendar.getInstance();
                    calendar.setTimeInMillis(System.currentTimeMillis());
                    
                    // Check if the time is tomorrow or not
                    boolean tomorrow = false;
                    if (calendar.get(Calendar.HOUR_OF_DAY) > hourOfDay) {
                    	tomorrow = true;
                    } else {
                    	if ((calendar.get(Calendar.HOUR_OF_DAY) == hourOfDay) &&
                    		(calendar.get(Calendar.MINUTE) >= minute)) {
                    		tomorrow = true;
                    	}
                    }
                    
                    // Set the alarm time
                    calendar.set(Calendar.HOUR_OF_DAY, hourOfDay);
                    calendar.set(Calendar.MINUTE, minute);   
                    calendar.set(Calendar.SECOND, 0);
                    if (tomorrow) {
                    	calendar.add(Calendar.HOUR, 24);
                    }

                    // Schedule the alarm!
                    AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
                    am.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), intent);

                    // Tell the user about what we did.
                    if (mToast != null) {
                        mToast.cancel();
                    }
                    mToast = Toast.makeText(AlarmController.this, R.string.one_shot_scheduled,
                            Toast.LENGTH_SHORT);
                    mToast.show();
                    
                    /* End Alarm Code */
                    
                    mHour = hourOfDay;
                    mMinute = minute;
                    if (mHour >= 12) {
                    	if (mHour != 12) {
                    		mHour = mHour - 12;
                    	}
                    	AMPM = "PM";
                    } else {
                    	if (mHour == 0) {
                    		mHour = 12;
                    	}
                    	AMPM = "AM";
                    }       

                    setMoodView(R.drawable.alarm_small, R.string.one_shot_scheduled);
                    alarmSet = true;
                    timer.cancel();
                    timer = new Timer();
                    timer.schedule(new TimerTask() {

                        public void run() {
                        	alarmSet = false;
                        	mNotificationManager.cancel(R.layout.alarm_controller);
                        	updateDisplay();
                        }
                    
                    },calendar.getTime());
                    updateDisplay();
                }
            };

    private static String pad(int c) {
        if (c >= 10)
            return String.valueOf(c);
        else
            return "0" + String.valueOf(c);
    }
    
    public void setAlarm(boolean status) {
    	alarmSet = status;
    }
    
    /* Use this if you don't want the extra notification
    private void setMood(int moodId, int textId, boolean showTicker) {
        // This is who should be launched if the user selects our notification.
        Intent contentIntent = new Intent(this, AlarmController.class);

        // This is who should be launched if the user selects the app icon in the notification.
        Intent appIntent = new Intent(this, FlashlightMenu.class);

        // In this sample, we'll use the same text for the ticker and the expanded notification
        CharSequence text = getText(textId);

        // choose the ticker text
        String tickerText = showTicker ? getString(textId) : null;

        mNotificationManager.notify(
                   R.layout.alarm_controller, // we use a string id because it is a unique
                                                      // number.  we use it later to cancel the
                                                      // notification
                   new Notification(
                       this,                        // our context
                       moodId,                      // the icon for the status bar
                       tickerText,                  // the text to display in the ticker
                       System.currentTimeMillis(),  // the timestamp for the notification
                       getText(R.string.status_bar_notification_title),
                                                    // the title for the notification
                       text,                        // the details to display in the notification
                       contentIntent,               // the contentIntent (see above)
                       R.drawable.alarm_small,  // the app icon
                       getText(R.string.status_bar_notification_title), // the name of the app
                       appIntent));                 // the appIntent (see above)
    } */

    
    private void setMoodView(int moodId, int textId) {
        // Instead of the normal constructor, we're going to use the one with no args and fill
        // in all of the data ourselves.  The normal one uses the default layout for notifications.
        // You probably want that in most cases, but if you want to do something custom, you
        // can set the expandedView field to your own RemoteViews object.
        Notification notif = new Notification();

        // This is who should be launched if the user selects our notification.
        notif.contentIntent = new Intent(this, AlarmController.class);

        // This is who should be launched if the user selects the app icon in the notification.
        notif.appIntent = new Intent(this, FlashlightMenu.class);

        // In this sample, we'll use the same text for the ticker and the expanded notification
        CharSequence text = getText(textId);
        notif.tickerText = text;

        // the icon for the status bar
        notif.icon = moodId;

        // our custom view
        RemoteViews expandedView = new RemoteViews(getPackageName(), R.layout.status_bar_balloon);
        expandedView.setTextViewText(R.id.text, text);
        expandedView.setImageViewResource(R.id.icon, moodId);
        notif.expandedView = expandedView;

        // we use a string id because is a unique number.  we use it later to cancel the
        // notification
        mNotificationManager.notify(R.layout.alarm_controller, notif);
    }    
    
    @Override
    protected void onResume() {
    	updateDisplay();
    	super.onResume();
    }

    
}