/* 
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.berkeley.cs160.theflash.flashlight;

import edu.berkeley.cs160.theflash.flashlight.R;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.*;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.widget.Toast;

public class Ambient extends Activity {

	private SampleView mAmbientView;
	
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        
        // Turn off the title bar
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        
        mAmbientView = new SampleView(this);
        setContentView(mAmbientView);
        
        //set onKeyListener
        mAmbientView.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		//Pressing the middle select key to switch to Spinner mode
        		if(keyCode==KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_UP){
        			//Notify flashlight off
        			Toast.makeText(Ambient.this, "Go Bears!", Toast.LENGTH_SHORT).show();
        			Intent intent = new Intent(Ambient.this, GoBears.class);
    	            startActivity(intent);
    	            finish();
        			return true;
        		}else{
        			return false;
        		}
        	}
        });
    }
    
    private static class SampleView extends View {
        private Paint mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        private ColorMatrix mCM = new ColorMatrix();
        private Bitmap mBitmap;
        private Bitmap mBitmap2;
        private float mAngle;

        public SampleView(Context context) {
            super(context);
            setFocusable(true);

            mBitmap = BitmapFactory.decodeResource(context.getResources(),
                                                   R.drawable.calbear);
            
            mBitmap2 = BitmapFactory.decodeResource(context.getResources(),
                    R.drawable.calbear_yellow);
        }
        
        @Override protected void onDraw(Canvas canvas) {
            Paint paint = mPaint;
            float x = 55;
            float y = 55;

            canvas.drawColor(Color.BLACK);

            paint.setColorFilter(null);
            canvas.drawBitmap(mBitmap, x, y, paint); //blue bear
            
            canvas.drawBitmap(mBitmap2, x + mBitmap.getWidth() + 10, y + mBitmap.getHeight() + 10, paint); //yellow bear

            mCM.setRGB2YUV();
            ColorMatrix cm = new ColorMatrix();
            cm.setRotate(0, mAngle);
            mCM.postConcat(cm);
            cm.setYUV2RGB();
            mCM.postConcat(cm);
            paint.setColorFilter(new ColorMatrixFilter(mCM));
            canvas.drawBitmap(mBitmap2, x + mBitmap2.getWidth() + 10, y, paint);
            mAngle += 10;
            if (mAngle > 180) {
                mAngle = -180;
            }
            
            mCM.setRGB2YUV();
            cm = new ColorMatrix();
            cm.setRotate(0, mAngle);
            mCM.postConcat(cm);
            cm.setYUV2RGB();
            mCM.postConcat(cm);
            paint.setColorFilter(new ColorMatrixFilter(mCM));
            canvas.drawBitmap(mBitmap, x, y + mBitmap.getHeight() + 10, paint);
            mAngle += 10;
            if (mAngle > 180) {
                mAngle = -180;
            }
            
            invalidate();
        }

        @Override public boolean onKeyDown(int keyCode, KeyEvent event) {
            switch (keyCode) {
                case KeyEvent.KEYCODE_S:
//                    mDoSat = !mDoSat;
                    invalidate();
                    return true;
            }
            return super.onKeyDown(keyCode, event);
        }
    }
}

