package edu.berkeley.cs160.theflash.flashlight;

/* 
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import edu.berkeley.cs160.theflash.flashlight.R;
import android.app.Activity;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.BaseAdapter;
import android.content.Intent;
import android.content.Context;
import android.view.ViewGroup;
import android.content.pm.ResolveInfo;
import android.widget.Gallery;
import android.widget.Toast;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.view.animation.*;
import android.graphics.*;
import java.util.List;


/**
 * This is the Flashlight app.
 */
public class Blinking extends Activity {
	private BlinkingView mBlinkingView;

    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        
        loadApps(); // do this in onresume?
        
        //Get PowerManager
        pm = (PowerManager) getSystemService("power");
        //Get WakeLock
        wLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "Flashlight");
        wLock.acquire();
        hasLock = true;
        
        //Set Flashlight
        currentFlashlight = this;
        
        //Set color pos
        colorPos = 0;
        
        //Set oldAlpha to full brightness.
        oldAlpha = 255;
        
        // Turn off the title bar
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        // Make our view
        setContentView(R.layout.blinking);
        mBlinkingView = (BlinkingView) findViewById(R.id.blinking_imageHolder);
        mBlinkingView.setAdapter(new AppsAdapter(this));
        
        blinking = (ImageView) findViewById(R.id.blinkinglight);
        
        highLow = new AlphaAnimation((float) 1.0, (float) 0.0);  
        currentSpeed = speed_1;
        highLow.setDuration(currentSpeed);
        highLow.setRepeatMode(Animation.REPEAT);
        bgColor = white;
        blinking.setBackgroundColor(bgColor);
        blinking.startAnimation(highLow);
        
        //Progress Bar (Opacity Bar in this case)
        progressHorizontal = (ProgressBar) findViewById(R.id.blinking_progress_horizontal);
        setProgress(progressHorizontal.getProgress() * 100);
        progressHorizontal.setBackground(R.drawable.gradient_100);
        
        //Toast Instructions
        //Toast.makeText(this, "Center - Cycle Speed\nUp/Down - Color\nLeft/Right - Brightness\nMenu - Help", Toast.LENGTH_SHORT).show();
        
        //set onKeyListener
        mBlinkingView.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		//Pressing Menu to get help Toast
                if(keyCode == KeyEvent.KEYCODE_SOFT_LEFT){
                	Toast.makeText(Blinking.currentFlashlight, "Center - cycle Speed\nUp/Down - Color\nLeft/Right - Brightness", Toast.LENGTH_LONG).show();
                }
        		
        		//Pressing the middle select button to cycle speeds.
        		if(keyCode==KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_UP){
        			if(currentSpeed == speed_1){
        				currentSpeed = speed_2;
        				Toast.makeText(Blinking.currentFlashlight, "Speed: Medium", Toast.LENGTH_SHORT).show();
        				animationUpdate(speed_2);
        				return true;
        			}else if(currentSpeed == speed_2){
        				currentSpeed = speed_3;
        				Toast.makeText(Blinking.currentFlashlight, "Speed: Fast", Toast.LENGTH_SHORT).show();
        				animationUpdate(speed_3);
        				return true;
        			}else if(currentSpeed == speed_3){
        				currentSpeed = speed_1;
        				Toast.makeText(Blinking.currentFlashlight, "Speed: Slow", Toast.LENGTH_SHORT).show();
        				animationUpdate(speed_1);
        				return true;
        			}else{
        				return false;
        			}
        		}
        		
        		//Moving brightness selector right
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_RIGHT && event.getAction() == KeyEvent.ACTION_DOWN){
        			
        			int progress = progressHorizontal.getProgress();
        			if(getBrightness(progress)=="0"){
        				progressHorizontal.setProgress(5);
        			}
        			progressHorizontal.incrementProgressBy(25);
        			
        			progress = progressHorizontal.getProgress(); //for 0 case
        			
        			//set slider button
        			setSlider(progress);
        			
        			highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
        	        highLow.setDuration(currentSpeed);
        	        highLow.setRepeatMode(Animation.REPEAT);
        	        blinking.setBackgroundColor(bgColor);
        	        blinking.startAnimation(highLow);
        	        
        			//Notify percent brightness
        			Toast.makeText(Blinking.currentFlashlight, "Brightness: "+getBrightness(progress)+"%", Toast.LENGTH_SHORT).show();
        			
        			return true;
        		}
        		//Moving brightness selector left
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_LEFT && event.getAction() == KeyEvent.ACTION_DOWN){
        			
        			progressHorizontal.incrementProgressBy(-25);
        			int progress = progressHorizontal.getProgress();
        			if(getBrightness(progress) == "0"){
        				progressHorizontal.setProgress(0);
        			}
        			
        			//set slider button
        			setSlider(progress);
    				
        			highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
        	        highLow.setDuration(currentSpeed);
        	        highLow.setRepeatMode(Animation.REPEAT);
        	        blinking.setBackgroundColor(bgColor);
        	        blinking.startAnimation(highLow);
        			
        			//Notify percent brightness
        			Toast.makeText(Blinking.currentFlashlight, "Brightness: "+getBrightness(progress)+"%", Toast.LENGTH_SHORT).show();
        			
        			return true;
        		}
        		//Moving color selector down
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_DOWN && event.getAction() == KeyEvent.ACTION_UP){

        			int progress = progressHorizontal.getProgress();
        			if(Blinking.colorPos == 0){
        				Blinking.colorPos = 1;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = red;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 1){
        				Blinking.colorPos = 2;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = orange;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 2){
        				Blinking.colorPos = 3;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = yellow;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 3){
        				Blinking.colorPos = 4;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = green;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 4){
        				Blinking.colorPos = 5;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = blue;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 5){
        				Blinking.colorPos = 6;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = indigo;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 6){
        				Blinking.colorPos = 7;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = violet;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 7){
        				//do nothing
        			}else{
        				//do nothing
        			}
        			
        			return true;
        		}
        		//Moving color selector up.
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_UP && event.getAction() == KeyEvent.ACTION_UP){
        			
        			int progress = progressHorizontal.getProgress();
        			if(Blinking.colorPos == 0){
        				//do nothing
        			}else if(Blinking.colorPos == 1){
        				Blinking.colorPos = 0;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = white;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 2){
        				Blinking.colorPos = 1;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = red;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 3){
        				Blinking.colorPos = 2;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = orange;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 4){
        				Blinking.colorPos = 3;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = yellow;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 5){
        				Blinking.colorPos = 4;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = green;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 6){
        				Blinking.colorPos = 5;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = blue;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else if(Blinking.colorPos == 7){
        				Blinking.colorPos = 6;
        				highLow = new AlphaAnimation((float) progress/255, (float) 0.0);               
            	        highLow.setDuration(currentSpeed);
            	        highLow.setRepeatMode(Animation.REPEAT);
            	        bgColor = indigo;
            	        blinking.setBackgroundColor(bgColor);
            	        blinking.startAnimation(highLow);
        			}else{
        				//do nothing
        			}
        			
        			return true;
        		}
        		//Release WakeLock when user presses back.
        		else if((keyCode==KeyEvent.KEYCODE_BACK || keyCode==KeyEvent.KEYCODE_CALL || 
        				keyCode == KeyEvent.KEYCODE_ENDCALL || keyCode==KeyEvent.KEYCODE_HOME ||
        				keyCode == KeyEvent.KEYCODE_POWER)
        				&& event.getAction() == KeyEvent.ACTION_DOWN && hasLock){
        			wLock.release();
        			hasLock = false;
        		}
                return false;
        	}
        });
    }
    
    private List<ResolveInfo> mApps;
    
    private void loadApps() {
        Intent mainIntent = new Intent(Intent.MAIN_ACTION, null);
        mainIntent.addCategory(Intent.LAUNCHER_CATEGORY);

        mApps = getPackageManager().queryIntentActivities(mainIntent, 0);
    }

    public class AppsAdapter extends BaseAdapter {
        public AppsAdapter(Context context) {
        }

        public View getView(int position, View convertView, ViewGroup parent) {
            ImageView i = new ImageView(Blinking.this);

            if(position == 0){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.white));
        	}else if(position == 1){
            	 i.setImageDrawable(getResources().getDrawable(R.drawable.red));
            }else if(position == 2){
	            i.setImageDrawable(getResources().getDrawable(R.drawable.orange));
            }else if(position == 3){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.yellow));
            }else if(position == 4){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.green));
            }else if(position == 5){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.blue));
            }else if(position == 6){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.indigo));
            }else if(position == 7){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.violet));
            }else{
            	i.setImageDrawable(getResources().getDrawable(R.drawable.white));
            }
            i.setScaleType(ImageView.ScaleType.FIT_CENTER);
            i.setLayoutParams(new Gallery.LayoutParams(15, 15));
            return i;
        }


        public final int getCount() {
            return 8;
        }

        public final Object getItem(int position) {
            return mApps.get(position);
        }

        public final long getItemId(int position) {
            return position;
        }
    }
    
    private void animationUpdate(long duration) {       
        highLow.setDuration(duration);       
        blinking.clearAnimation();
        blinking.startAnimation(highLow);
    }
    
    private String getBrightness(int progress){
    	int percent = (progress*100/255);
    	if(percent%10!=0){
    		percent --;
    	}
    	return Integer.toString(percent);
    }
    
    private void setSlider(int progress){
    	int percent = (progress*100/255);
    	if(percent %10 !=0){
    		percent --;
    	}
    	switch(percent){
    	case 0: progressHorizontal.setBackground(R.drawable.gradient_0); break;
    	case 10: progressHorizontal.setBackground(R.drawable.gradient_10); break;
    	case 20: progressHorizontal.setBackground(R.drawable.gradient_20); break;
    	case 30: progressHorizontal.setBackground(R.drawable.gradient_30); break;
    	case 40: progressHorizontal.setBackground(R.drawable.gradient_40); break;
    	case 50: progressHorizontal.setBackground(R.drawable.gradient_50); break;
    	case 60: progressHorizontal.setBackground(R.drawable.gradient_60); break;
    	case 70: progressHorizontal.setBackground(R.drawable.gradient_70); break;
    	case 80: progressHorizontal.setBackground(R.drawable.gradient_80); break;
    	case 90: progressHorizontal.setBackground(R.drawable.gradient_90); break;
    	case 100: progressHorizontal.setBackground(R.drawable.gradient_100); break;
    	default: progressHorizontal.setBackground(R.drawable.gradient_100); break;
    	}
    }
    
    protected void onPause(){
    	if(hasLock){
    		wLock.release();
    		hasLock = false;
    	}
    	super.onPause();
    }
    
    protected static Activity currentFlashlight; //this Flashlight
    protected static int colorPos;  //Current Background color, 0 is white, 1-7 is ROYGBIV
    protected static int oldAlpha; //Previous alpha setting when light was turned off.
    protected static ProgressBar progressHorizontal; //Progress Bar
    protected static PowerManager pm; //Allows us to keep screen on
    protected static WakeLock wLock;
    protected static boolean hasLock; //true if has the WakeLock
    protected AlphaAnimation highLow;
    private ImageView blinking;
    private int bgColor;
    private int currentSpeed; // speed of blinking from 1-3.
    
    //Speeds
    private int speed_1 = 2000;
    private int speed_2 = 500;
    private int speed_3 = 250;
    
    //Colors
    private final int white = Color.WHITE;
    private final int red = Color.argb(255, 226, 0, 1);
    private final int orange = Color.argb(255, 226, 139, 0);
    private final int yellow = Color.argb(255, 255, 246, 1);
    private final int green = Color.argb(255, 51, 218, 6);
    private final int blue = Color.argb(255, 0, 60, 255);
    private final int indigo = Color.argb(255, 137, 0, 254);
    private final int violet = Color.argb(255, 192, 1, 255);
}