package edu.berkeley.cs160.theflash.flashlight;

/* 
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import edu.berkeley.cs160.theflash.flashlight.R;
import android.app.Activity;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.BaseAdapter;
import android.content.Intent;
import android.content.Context;
import android.view.ViewGroup;
import android.content.pm.ResolveInfo;
import android.widget.Gallery;
import android.widget.Toast;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;

import java.util.List;


/**
 * This is the Flashlight app.
 */
public class Flashlight extends Activity {
	private FlashlightView mFlashlightView;

    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        
        loadApps(); // do this in onresume?
        
        //Get PowerManager
        pm = (PowerManager) getSystemService("power");
        //Get WakeLock
        wLock = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "Flashlight");
        wLock.acquire();
        hasLock = true;
        
        //Set Flashlight
        currentFlashlight = this;
        
        //Set color pos
        colorPos = 0;
        
        //Set light to being on.
        lightOn = true;
        
        //Set oldAlpha to full brightness.
        oldAlpha = 255;
        
        // Turn off the title bar
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        // Make our view
        setContentView(R.layout.flashlight);
        mFlashlightView = (FlashlightView) findViewById(R.id.imageHolder);
        mFlashlightView.setAdapter(new AppsAdapter(this));
        
        //Progress Bar (Opacity Bar in this case)
        progressHorizontal = (ProgressBar) findViewById(R.id.progress_horizontal);
        setProgress(progressHorizontal.getProgress() * 100);
        progressHorizontal.setBackground(R.drawable.gradient_100);
        
        //Toast Instructions
        //Toast.makeText(Flashlight.currentFlashlight, "Center - On/Off\nUp/Down - Color\nLeft/Right - Brightness\nMenu - Help", Toast.LENGTH_SHORT).show();
        
        //set onKeyListener
        mFlashlightView.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		
        		//Pressing Menu to get help Toast
                if(keyCode == KeyEvent.KEYCODE_SOFT_LEFT){
                	Toast.makeText(Flashlight.currentFlashlight, "Center - On/Off\nUp/Down - Color\nLeft/Right - Brightness", Toast.LENGTH_LONG).show();
                }
                
        		//Pressing the middle select button to turn light on or off.
        		if(keyCode==KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_UP){
        			if(lightOn == true){
        				lightOn = false;
        				Drawable drawableImage = mFlashlightView.getBackground();
        				oldAlpha = progressHorizontal.getProgress();
            			drawableImage.setAlpha(0);
            			mFlashlightView.setBackground(drawableImage);
            			
            			//Notify flashlight off
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight Off", Toast.LENGTH_SHORT).show();
            			
            			return true;
        			}else{
        				lightOn = true;
        				Drawable drawableImage = mFlashlightView.getBackground();
        				drawableImage.setAlpha(oldAlpha);
        				mFlashlightView.setBackground(drawableImage);
        				
        				//Notify flashlight on
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight On", Toast.LENGTH_SHORT).show();
            			
            			return true;
        				
        			}
        		}
        		
        		//Moving brightness selector right
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_RIGHT && event.getAction() == KeyEvent.ACTION_DOWN){
        			if(lightOn == false){
        				lightOn = true;
        				//Notify flashlight on
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight On", Toast.LENGTH_SHORT).show();
        			}
        			
        			int progress = progressHorizontal.getProgress();
        			if(getBrightness(progress)=="0"){
        				progressHorizontal.setProgress(5);
        			}
        			progressHorizontal.incrementProgressBy(25);
        			
        			progress = progressHorizontal.getProgress(); //for 0 case
    				Drawable drawableImage = mFlashlightView.getBackground();
        			drawableImage.setAlpha(progress);
        			mFlashlightView.setBackground(drawableImage);
        			
        			//set slider button
        			setSlider(progress);
        			
        			//Notify percent brightness
        			Toast.makeText(Flashlight.currentFlashlight, "Brightness: "+getBrightness(progress)+"%", Toast.LENGTH_SHORT).show();
        			
        			return true;
        		}
        		//Moving brightness selector left
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_LEFT && event.getAction() == KeyEvent.ACTION_DOWN){
        			if(lightOn == false){
        				lightOn = true;
        				//Notify flashlight on
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight On", Toast.LENGTH_SHORT).show();
        			}
        			
        			progressHorizontal.incrementProgressBy(-25);
        			int progress = progressHorizontal.getProgress();
        			if(getBrightness(progress) == "0"){
        				progressHorizontal.setProgress(0);
        			}
    				Drawable drawableImage = mFlashlightView.getBackground();
        			drawableImage.setAlpha(progress);
        			mFlashlightView.setBackground(drawableImage);
        			
        			//set slider button
        			setSlider(progress);
        			
        			//Notify percent brightness
        			Toast.makeText(Flashlight.currentFlashlight, "Brightness: "+getBrightness(progress)+"%", Toast.LENGTH_SHORT).show();
        			
        			return true;
        		}
        		//Moving color selector down
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_DOWN && event.getAction() == KeyEvent.ACTION_UP){
        			if(lightOn == false){
        				lightOn = true;
        				//Notify flashlight on
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight On", Toast.LENGTH_SHORT).show();
        			}
        			int progress = progressHorizontal.getProgress();
        			if(Flashlight.colorPos == 0){
        				Flashlight.colorPos = 1;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_red);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 1){
        				Flashlight.colorPos = 2;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_orange);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 2){
        				Flashlight.colorPos = 3;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_yellow);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 3){
        				Flashlight.colorPos = 4;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_green);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 4){
        				Flashlight.colorPos = 5;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_blue);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 5){
        				Flashlight.colorPos = 6;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_indigo);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 6){
        				Flashlight.colorPos = 7;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_violet);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 7){
        				//do nothing
        			}else{
        				//do nothing
        			}
        			
        			return true;
        		}
        		//Moving color selector up.
        		else if(keyCode==KeyEvent.KEYCODE_DPAD_UP && event.getAction() == KeyEvent.ACTION_UP){
        			if(lightOn == false){
        				lightOn = true;
        				//Notify flashlight on
            			Toast.makeText(Flashlight.currentFlashlight, "Flashlight On", Toast.LENGTH_SHORT).show();
        			}
        			int progress = progressHorizontal.getProgress();
        			if(Flashlight.colorPos == 0){
        				//do nothing
        			}else if(Flashlight.colorPos == 1){
        				Flashlight.colorPos = 0;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_white);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 2){
        				Flashlight.colorPos = 1;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_red);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 3){
        				Flashlight.colorPos = 2;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_orange);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 4){
        				Flashlight.colorPos = 3;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_yellow);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 5){
        				Flashlight.colorPos = 4;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_green);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 6){
        				Flashlight.colorPos = 5;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_blue);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else if(Flashlight.colorPos == 7){
        				Flashlight.colorPos = 6;
        				Drawable newBackground = getResources().getDrawable(R.drawable.flashlight_indigo);
        				newBackground.setAlpha(progress);
        				mFlashlightView.setBackground(newBackground);
        			}else{
        				//do nothing
        			}
        			
        			return true;
        		}
        		//Release WakeLock when user presses back.
        		else if((keyCode==KeyEvent.KEYCODE_BACK || keyCode==KeyEvent.KEYCODE_CALL || 
        				keyCode == KeyEvent.KEYCODE_ENDCALL || keyCode==KeyEvent.KEYCODE_HOME ||
        				keyCode == KeyEvent.KEYCODE_POWER)
        				&& event.getAction() == KeyEvent.ACTION_DOWN && hasLock){
        			wLock.release();
        			hasLock = false;
        		}
                return false;
        	}
        });
    }
    
    private List<ResolveInfo> mApps;
    
    private void loadApps() {
        Intent mainIntent = new Intent(Intent.MAIN_ACTION, null);
        mainIntent.addCategory(Intent.LAUNCHER_CATEGORY);

        mApps = getPackageManager().queryIntentActivities(mainIntent, 0);
    }

    public class AppsAdapter extends BaseAdapter {
        public AppsAdapter(Context context) {
        }

        public View getView(int position, View convertView, ViewGroup parent) {
            ImageView i = new ImageView(Flashlight.this);

            if(position == 0){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.white));
        	}else if(position == 1){
            	 i.setImageDrawable(getResources().getDrawable(R.drawable.red));
            }else if(position == 2){
	            i.setImageDrawable(getResources().getDrawable(R.drawable.orange));
            }else if(position == 3){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.yellow));
            }else if(position == 4){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.green));
            }else if(position == 5){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.blue));
            }else if(position == 6){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.indigo));
            }else if(position == 7){
            	i.setImageDrawable(getResources().getDrawable(R.drawable.violet));
            }else{
            	i.setImageDrawable(getResources().getDrawable(R.drawable.white));
            }
            i.setScaleType(ImageView.ScaleType.FIT_CENTER);
            i.setLayoutParams(new Gallery.LayoutParams(15, 15));
            return i;
        }


        public final int getCount() {
            return 8;
        }

        public final Object getItem(int position) {
            return mApps.get(position);
        }

        public final long getItemId(int position) {
            return position;
        }
    }
    
    private String getBrightness(int progress){
    	int percent = (progress*100/255);
    	if(percent%10!=0){
    		percent --;
    	}
    	return Integer.toString(percent);
    }
    
    private void setSlider(int progress){
    	int percent = (progress*100/255);
    	if(percent %10 !=0){
    		percent --;
    	}
    	switch(percent){
    	case 0: progressHorizontal.setBackground(R.drawable.gradient_0); break;
    	case 10: progressHorizontal.setBackground(R.drawable.gradient_10); break;
    	case 20: progressHorizontal.setBackground(R.drawable.gradient_20); break;
    	case 30: progressHorizontal.setBackground(R.drawable.gradient_30); break;
    	case 40: progressHorizontal.setBackground(R.drawable.gradient_40); break;
    	case 50: progressHorizontal.setBackground(R.drawable.gradient_50); break;
    	case 60: progressHorizontal.setBackground(R.drawable.gradient_60); break;
    	case 70: progressHorizontal.setBackground(R.drawable.gradient_70); break;
    	case 80: progressHorizontal.setBackground(R.drawable.gradient_80); break;
    	case 90: progressHorizontal.setBackground(R.drawable.gradient_90); break;
    	case 100: progressHorizontal.setBackground(R.drawable.gradient_100); break;
    	default: progressHorizontal.setBackground(R.drawable.gradient_100); break;
    	}
    }
    
    protected void onPause(){
    	if(hasLock){
    		wLock.release();
    		hasLock = false;
    	}
    	super.onPause();
    }
    
    protected static Activity currentFlashlight; //this Flashlight
    protected static int colorPos;  //Current Background color, 0 is white, 1-7 is ROYGBIV
    protected static boolean lightOn; //true if light is on.
    protected static int oldAlpha; //Previous alpha setting when light was turned off.
    protected static PowerManager pm; //Allows us to keep screen on
    protected static WakeLock wLock;
    protected static ProgressBar progressHorizontal; //Progress Bar
    protected static boolean hasLock; //true if has the WakeLock
    protected static boolean justStarted; //true if this is the first time the application has been run.
}