/* 
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package edu.berkeley.cs160.theflash.flashlight;

// Need the following import to get access to the app resources, since this
// class is in a sub-package.
//import com.google.android.samples.R;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.*;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.Window;
import android.widget.Toast;

public class Spinner extends Activity {

	SampleView mSpinnerView;
	
    @Override
    protected void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        
        // Turn off the title bar
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        
        mSpinnerView = new SampleView(this);
        setContentView(mSpinnerView);
        
        //set onKeyListener
        mSpinnerView.setOnKeyListener(new View.OnKeyListener(){
        	
        	public boolean onKey(View v, int keyCode, KeyEvent event){
        		//Pressing the middle select key to switch to Bear mode
        		if(keyCode==KeyEvent.KEYCODE_DPAD_CENTER && event.getAction() == KeyEvent.ACTION_UP){
        			//Notify flashlight off
        			Toast.makeText(Spinner.this, "Bears", Toast.LENGTH_SHORT).show();
        			Intent intent = new Intent(Spinner.this, Ambient.class);
    	            startActivity(intent);
    	            finish();
        			return true;
        		}else{
        			return false;
        		}
        	}
        });
    }
    
    private static class SampleView extends View {
        private Paint mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        private float mRotate;
        private Matrix mMatrix = new Matrix();
        private Shader mShader;
        private boolean mDoTiming;

        public SampleView(Context context) {
            super(context);
            setFocusable(true);

            float x = 160;
            float y = 110;
            mShader = new SweepGradient(x, y, new int[] {Color.GREEN,
            									  Color.YELLOW,
                                                  Color.RED,
                                                  Color.MAGENTA,
                                                  Color.BLUE,
                                                  Color.CYAN,
                                                  Color.GREEN }, null);
            mPaint.setShader(mShader);
        }
        
        @Override protected void onDraw(Canvas canvas) {
            Paint paint = mPaint;
            float x = 160;
            float y = 110;

            canvas.drawColor(Color.BLACK);

            mMatrix.setRotate(mRotate, x, y);
            mShader.setLocalMatrix(mMatrix);
            mRotate += 3;
            if (mRotate >= 360) {
                mRotate = 0;
            }
            invalidate();

            if (mDoTiming) {
                long now = System.currentTimeMillis();
                for (int i = 0; i < 20; i++) {
                    canvas.drawCircle(x, y, 80, paint);
                }
                now = System.currentTimeMillis() - now;
                android.util.Log.d("skia", "sweep ms = " + (now/20.));
            }
            else {
                canvas.drawCircle(x, y, 80, paint);
            }
        }

        @Override public boolean onKeyDown(int keyCode, KeyEvent event) {
            switch (keyCode) {
                case KeyEvent.KEYCODE_D:
                    mPaint.setDither(!mPaint.isDither());
                    invalidate();
                    return true;
                case KeyEvent.KEYCODE_T:
                    mDoTiming = !mDoTiming;
                    invalidate();
                    return true;
            }
            return super.onKeyDown(keyCode, event);
        }
    }
}

